//Copyright (c) 1999, Art Gittleman
//This example is provided WITHOUT ANY WARRANTY either expressed or implied.
// Prepared by Dr. Spiegel

// Misuse a buffer to cause two threads to be stuck on wait()

/* Simple example to illustrate deadlock.
 */
  
public class Deadlock {
  public static final int size = 3;   	// works if size is 1
  class Buffer {
    int[] buffer = new int [size];      // the data       
    int putpos=0;                       // next position to put a value
    int getpos=0;                       // next position to get a value
    int number=0;                       // number of items in the buffer

    public synchronized void put(int value) throws InterruptedException {
      if (number != 0) {
        System.out.println("Cannot put -- Buffer not empty");
        wait();
      }
      number++;
      buffer[putpos] = value;
      System.out.println("Put "+value);
      putpos = (putpos+1)%size;
      if (number == 1) notify();   // only notifies after 1st element inserted
    }

    public synchronized int get() throws InterruptedException {
      if (number != size) {
         System.out.println("Cannot get -- Buffer not full");
         wait();
      }
      number--;
      int n = buffer[getpos];
      System.out.println("Get "+n);
      getpos = (getpos+1)%size;
      if (number == size-1) notify();  // only notifies after first element removed
      return n;
    }
  }

  class Producer extends Thread {
     Buffer buf;
     int time;
     Producer(Buffer b, int t) { 
        buf = b;
        time = t;
     }			

     public void run() {
       for(int i=1; i<=10; i++)
         try {
             buf.put(i);
             sleep(time);
         }catch (InterruptedException e){
             e.printStackTrace();
         }
     }
  }

  class Consumer extends Thread {
     Buffer buf;
     int time;
     Consumer(Buffer b, int t) { 
       buf = b;
       time = t;
     }
     public void run() {
       for (int i=1; i<=10;i++)
         try {
           buf.get();
           sleep(time);
         }catch (InterruptedException e) {
               e.printStackTrace();
         }
     }
  }

  public static void main(String[] args) {
    Deadlock dl = new Deadlock();
    Buffer b = dl.new Buffer();
    Producer p = dl.new Producer(b,Integer.parseInt(args[0]));
    Consumer c = dl.new Consumer(b,Integer.parseInt(args[1]));
    p.start();
    c.start();
  }
}

